import greenfoot.GreenfootImage;
import java.util.ArrayList;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics2D;
import java.awt.RenderingHints;
import java.awt.Color;

/**
 * ChatRecord
 * 
 * @author Taylor Born
 * @version January 2014
 */
public class ChatRecord extends WindowComponent
{
    private static final int FONT_SIZE = 12;
    private static final Font FONT = new Font("Helvetica", Font.PLAIN, FONT_SIZE);

    private GreenfootImage image;
    private ArrayList<String> lines = new ArrayList<String>();
    private ArrayList<String> properties = new ArrayList<String>();
    private int scrollValue; // Height of content not showing in pixels that is above the top border
    private ScrollingListener scroller = initializeScroller();

    public ChatRecord(int width, int height)
    {
        image = new GreenfootImage(width, height);
        setImage(image);
    }
    public void record(String text, boolean script, String author, boolean user)
    {
        Font font = FONT;
        
        Graphics2D g = image.getAwtImage().createGraphics();
        g.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
        FontMetrics fm = g.getFontMetrics(font);
        
        String currentStr = author + ":";
        int currentWidth = fm.charsWidth(currentStr.toCharArray(), 0, currentStr.length());
        
        g.dispose();
        
        String textBodyProperties = ",c" + Color.WHITE.getRGB() + "," + (script ? currentStr.length() + ",i," : "") + ";";
        String currentProperties = "0,c" + (user ? Color.GREEN.getRGB() : Color.ORANGE.getRGB()) + ",;" + currentStr.length() + "," + textBodyProperties;
        
        if (script)
        {
            font = font.deriveFont(Font.ITALIC);
            text = "\"" + text + "\"";
        }
        
        add(text, currentStr, currentWidth, font, textBodyProperties, currentProperties);
    }
    public void record(String text)
    {
        String props = ",c" + Color.RED.getRGB() + ",;";
        add(text, "", 0, FONT, props, "0" + props);
    }
    private void add(String text, String currentStr, int currentWidth, Font font, String textBodyProperties, String currentProperties)
    {
        final int WIDTH_FOR_LINES = 185;
        
        text += ' ';
        
        Graphics2D g = image.getAwtImage().createGraphics();
        g.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
        FontMetrics fm = g.getFontMetrics(font);
        
        while (true)
        {
            int spaceIndex = text.indexOf(' ');
            
            String newStr = (currentStr.isEmpty() ? "" : " ") + text.substring(0, spaceIndex);
            
            int w = fm.charsWidth(newStr.toCharArray(), 0, newStr.length());
            
            if (currentWidth + w < WIDTH_FOR_LINES)
            {
                currentStr += newStr;
                currentWidth += w;
                text = text.substring(spaceIndex + 1);
                if (text.isEmpty())
                {
                    lines.add(currentStr);
                    properties.add(currentProperties);
                    lines.add(null);
                    properties.add(null);
                    break;
                }
            }
            else if (currentWidth != 0)
            {
                lines.add(currentStr);
                properties.add(currentProperties);
                currentStr = "";
                currentProperties = 0 + textBodyProperties;
                currentWidth = 0;
            }
            else
            {
                int i;
                for (i = 1; i < newStr.length(); i++)
                    if (fm.charsWidth(newStr.substring(0, i).toCharArray(), 0, i) >= WIDTH_FOR_LINES)
                        break;
                lines.add(newStr.substring(0, i - 1));
                properties.add(currentProperties);
                text = text.substring(i - 1);
            }
        }
        
        g.dispose();
        scrollToBotton();
        updateImage();
    }
    private void scrollToBotton()
    {
        int contentHeight = lines.size() * (FONT_SIZE + 2);
        if (contentHeight >= image.getHeight())
            scrollValue = contentHeight - image.getHeight();
    }
    
    private void updateImage()
    {
        image.clear();
        image.setColor(backColor);
        image.fill();
        
        Font font;
        
        Graphics2D g = image.getAwtImage().createGraphics();
        g.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
        
        int startLine = scrollValue / (FONT_SIZE + 2);
        int offset = scrollValue % (FONT_SIZE + 2);
        for (int line = 0; startLine + line < lines.size() && -offset + line * (FONT_SIZE + 2) < image.getHeight(); line++)
        {
            String lineText = lines.get(startLine + line);
            if (lineText == null)
                continue;
            
            g.setFont(FONT);
            int widthOfDrawn = 0;
            String data = properties.get(startLine + line);
            int indexOfRemaining = 0;
            int indexOfNextChange = 0;//(data.isEmpty() ? -1 : Integer.parseInt(data.substring(0, data.indexOf(','))));
            
            while (indexOfRemaining != -1)
            {
                while (indexOfRemaining == indexOfNextChange)
                {
                    data = data.substring(data.indexOf(',') + 1);
                    while (data.charAt(0) != ';')
                    {
                        switch (data.charAt(0))
                        {
                            case 'c':
                                g.setColor(new Color(Integer.parseInt(data.substring(1, data.indexOf(',')))));
                                break;
                            case 'i':
                                g.setFont(g.getFont().deriveFont(Font.ITALIC));
                        }
                        data = data.substring(data.indexOf(',') + 1);
                    }
                    data = data.substring(1);
                    indexOfNextChange = (data.isEmpty() ? -1 : Integer.parseInt(data.substring(0, data.indexOf(','))));
                }
                
                String textToDraw = lineText.substring(indexOfRemaining, (indexOfNextChange == -1 ? lineText.length() : indexOfNextChange));
                g.drawString(textToDraw, 2 + widthOfDrawn, -offset + (line + 1) * (FONT_SIZE + 2));
                widthOfDrawn += g.getFontMetrics().charsWidth(textToDraw.toCharArray(), 0, textToDraw.length());
                indexOfRemaining = indexOfNextChange;
            }
        }
        
        g.dispose();
    }
    
    public void clear()
    {
        image.clear();
        lines.clear();
        properties.clear();
        scrollValue = 0;
    }
    
    @Override
    public void act()
    {
        super.act();
        
        // Listen for mouse scroll wheel
        int n = scroller.getScroll();
        
        int contentHeight = lines.size() * (FONT_SIZE + 2);
        if (contentHeight >= image.getHeight())
        {
            // If mouse is over this
            if (mouseOverThis() && inWorld())
            {
                // Add scroll amount from scroll wheel
                scrollValue += n;
                
                // Adjust scroll amount if out of bounds
                if (scrollValue < 0)
                    scrollValue = 0;
                else if (scrollValue > contentHeight - image.getHeight())
                    scrollValue = contentHeight - image.getHeight();
                updateImage();
            }
        }
    }
}