import greenfoot.World;
import java.util.ArrayList;
import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.io.IOException;

/**
 * Map
 * 
 * @author Taylor Born
 * @version January 2014
 */
public class Map
{
    private String name;
    private ArrayList<String> data = new ArrayList<String>();
    private char[][] tiles = new char[100][];
    private int startX, startY;
    
    public Map(BufferedReader br, String str) throws IOException
    {
        name = str.substring(1);
        int row = 0;
        str = br.readLine();
        int[] pair = retrievePair(str.substring("Start@".length()));
        startX = pair[0];
        startY = pair[1];
        
        str = br.readLine();
        while (str.charAt(0) != ' ')
        {
            tiles[row++] = str.substring(1).toCharArray();
            str = br.readLine();
        }
        
        str = br.readLine();
        while (!str.isEmpty() && str.charAt(0) == '-')
        {
            data.add(str.substring(1));
            str = br.readLine();
        }
    }
//     public int getCellColForX(int x)
//     {
//        return (startX * 30 + 15 + x) / 30; 
//     }
//     public int getCellRowForY(int y)
//     {
//        return (startY * 30 + 15 + y) / 30; 
//     }
    private char getFor(int x, int y)
    {
        int xx = startX * 30 + 15 + x;
        if (xx < 0)
            return ' ';
        xx /= 30;
        int yy = startY * 30 + 15 + y;
        if (yy < 0)
            return ' ';
        yy /= 30;
        
        if (yy >= tiles.length || tiles[yy] == null || xx >= tiles[yy].length)
            return ' ';
        return tiles[yy][xx];
    }
    public boolean isOut(int x, int y)
    {
        return getFor(x, y) == ' ';
    }
    
    public boolean isWater(int x, int y)
    {
        char ch = getFor(x, y);
        return ch == 'w' || ch == 'm';
    }
    
    /**
     * @return First two ints found in String in positions of array 0 and 1. Position 2 contains index of rest of String.
     */
    private int[] retrievePair(String s)
    {
        int[] ints = new int[3];
        ints[2] = s.indexOf(',');
        ints[0] = Integer.parseInt(s.substring(0, ints[2]));
        s = s.substring(++ints[2]);
        int commaIndex = s.indexOf(',');
        ints[1] = Integer.parseInt(commaIndex != -1 ? s.substring(0, commaIndex) : s);
        ints[2] += (commaIndex == -1 ? s.length() : commaIndex + 1);
        return ints;
    }
    
    
    public void load(World w)
    {
        for (int r = 0; r < tiles.length && tiles[r] != null; r++)
            for (int c = 0; c < tiles[r].length; c++)
                if (tiles[r][c] != ' ')
                {
                    if (c == 0 || tiles[r][c - 1] == ' ') // If should place fence to the left
                        w.addObject(new Fence(true), (c - startX) * 30 + Terrain.CENTER_PLAYER_X - 15, (r - startY) * 30 + Terrain.CENTER_PLAYER_Y);
                    if (c == tiles[r].length - 1 || tiles[r][c + 1] == ' ') // If should place fence to the right
                        w.addObject(new Fence(true), (c - startX) * 30 + Terrain.CENTER_PLAYER_X + 15, (r - startY) * 30 + Terrain.CENTER_PLAYER_Y);
                    if (r == 0 || tiles[r - 1].length <= c || tiles[r - 1][c] == ' ') // If should place fence above
                        w.addObject(new Fence(false), (c - startX) * 30 + Terrain.CENTER_PLAYER_X, (r - startY) * 30 + Terrain.CENTER_PLAYER_Y - 15);
                    if (r == tiles.length - 1 || tiles[r + 1] == null || tiles[r + 1].length <= c || tiles[r + 1][c] == ' ') // If should place fence below
                        w.addObject(new Fence(false), (c - startX) * 30 + Terrain.CENTER_PLAYER_X, (r - startY) * 30 + Terrain.CENTER_PLAYER_Y + 15);
                    
                    switch (tiles[r][c])
                    {
                        case 'k':
                        case 'e':
                            w.addObject(new Soldier(), (c - startX) * 30 + Terrain.CENTER_PLAYER_X, (r - startY) * 30 + Terrain.CENTER_PLAYER_Y);
                            break;
                        case 'm':
                        case 'w':
                            w.addObject(new Water(), (c - startX) * 30 + Terrain.CENTER_PLAYER_X, (r - startY) * 30 + Terrain.CENTER_PLAYER_Y);
                    }
                }
        for (String str : data)
            if (str.startsWith("Door"))
            {
                str = str.substring("Door".length());
                if (str.startsWith("Panel"))
                {
                    char doorID = str.charAt("Panel".length());
                    str = str.substring(str.indexOf(',') + 1);
                    
                    int[] pair = retrievePair(str); // Contains x and y
                    w.addObject(new GateAccess(doorID), (pair[0] - startX) * 30 + Terrain.CENTER_PLAYER_X, (pair[1] - startY) * 30 + Terrain.CENTER_PLAYER_Y);
                }
                else
                {
                    char doorID;
                    boolean bossDoor = str.startsWith("Boss");
                    if (bossDoor)
                        doorID = '#';
                    else
                        doorID = str.charAt(0);
                    
                    str = str.substring(str.indexOf(',') + 1);
                    int[] pair = retrievePair(str); // Contains x and y
                    str = str.substring(pair[2]);
                    // str now contains direction
                    boolean vertical = str.equals("west") || str.equals("east");
                    w.addObject(new Gate(doorID, vertical), (pair[0] - startX) * 30 + Terrain.CENTER_PLAYER_X + (!vertical ? 0 : str.equals("west") ? -15 : 15), (pair[1] - startY) * 30 + Terrain.CENTER_PLAYER_Y + (vertical ? 0 : str.equals("north") ? -15 : 15));
                }
            }
            else if (str.startsWith("DropKey"))
            {
                char keyID = str.charAt("DropKey".length());
                str = str.substring(str.indexOf(',') + 1);
                int[] pair = retrievePair(str); // Contains x and y
                
                Soldier soldier = (Soldier)w.getObjectsAt((pair[0] - startX) * 30 + Terrain.CENTER_PLAYER_X, (pair[1] - startY) * 30 + Terrain.CENTER_PLAYER_Y, Soldier.class).get(0);
                soldier.giveKey(keyID);
            }
            else if (str.startsWith("Pattern"))
            {
                boolean display = str.startsWith("PatternDisplay");
                char id = str.charAt(display ? "PatternDisplay".length() : "PatternInput".length());
                str = str.substring(str.indexOf(',') + 1);
                int[] pair = retrievePair(str);
                PatternConsole console;
                if (display)
                    console = new PatternConsole(id);
                else
                {
                    str = str.substring(pair[2]);
                    char doorID = str.charAt("Door".length());
                    console = new PatternConsole(id, doorID);
                }
                w.addObject(console, (pair[0] - startX) * 30 + Terrain.CENTER_PLAYER_X, (pair[1] - startY) * 30 + Terrain.CENTER_PLAYER_Y);
            }
            else if (str.startsWith("Spotlight"))
            {
                
            }
            else if (str.startsWith("Transport"))
            {
                char id = str.charAt("Transport".length());
                str = str.substring(str.indexOf(',') + 1);
                int[] pair = retrievePair(str); // Contains x and y
                str = str.substring(pair[2]);
                int[] pair2 = retrievePair(str); // Contains offset x and y, both as number of tiles
                w.addObject(new Transport(id, pair2[0], pair2[1]), (pair[0] - startX) * 30 + Terrain.CENTER_PLAYER_X, (pair[1] - startY) * 30 + Terrain.CENTER_PLAYER_Y);
            }
            else if (str.startsWith("PressurePlate"))
            {
                char id = str.charAt("PressurePlate".length());
                str = str.substring(str.indexOf(',') + 1);
                int[] pair = retrievePair(str); // Contains x and y
                w.addObject(new PressurePlate(id), (pair[0] - startX) * 30 + Terrain.CENTER_PLAYER_X, (pair[1] - startY) * 30 + Terrain.CENTER_PLAYER_Y);
            }
    }
}