import greenfoot.GreenfootImage;
import java.util.List;
import java.awt.Point;

/**
 * PatternConsole
 * 
 * @author Taylor Born
 * @version January 2014
 */
public class PatternConsole extends GameObject
{
    private static final GreenfootImage IMAGE = new GreenfootImage("PatternConsole.png");

    private char id;
    private char doorID; // 0 if this PatternConsole is a display
    private PatternTile[][] tiles = new PatternTile[3][3];
    private int[][] solution;
    private boolean unlocked;
    private Button leaveBtn = new Button("Leave", new Point(42, 23));

    public PatternConsole(char id)
    {
        this.id = id;
        solution = Terrain.main.getPattern(id);
        setImage(IMAGE);
    }
    public PatternConsole(char id, char doorID)
    {
        this(id);
        this.doorID = doorID;
    }
    
    @Override
    public void act()
    {
        if (doorID == 0)
        {
            if (tiles[0][0] == null || tiles[0][0].getWorld() == null)
                listenToActivate(true);
            listenToLeave();
            return;
        }
        
        if (unlocked)
            return;
        if (tiles[0][0] == null || tiles[0][0].getWorld() == null)
            listenToActivate(false);
        else if (tiles[0][0].getImage().getTransparency() == 255)
        {
            boolean correct = true;
outer:      for (int r = 0; r < 3; r++)
                for (int c = 0; c < 3; c++)
                    if (tiles[r][c].getState() != solution[r][c])
                    {
                        correct = false;
                        break outer;
                    }
            if (correct)
            {
                leaveBtn.removeFromWorld();
                for (int r = 0; r < 3; r++)
                    for (int c = 0; c < 3; c++)
                        tiles[r][c].lock();
                for (Gate g : (List<Gate>)getWorld().getObjects(Gate.class))
                    if (g.getID() == doorID)
                    {
                        getWorld().removeObject(g);
                        break;
                    }
                unlocked = true;
            }
            else
                listenToLeave();
        }
    }
    
    private void listenToLeave()
    {
        if (leaveBtn.wasClicked())
        {
            leaveBtn.removeFromWorld();
            for (int r = 0; r < 3; r++)
                for (int c = 0; c < 3; c++)
                    tiles[r][c].fadeOut();
        }
    }
    
    private void listenToActivate(boolean display)
    {
        if (onThis(Terrain.main.getPlayer()) && KeysPressed.testKey("e"))
        {
            getWorld().addObject(leaveBtn, getWorld().getWidth() / 2, getWorld().getHeight() / 2 + (PatternTile.SIZE + 1) * 2);
            leaveBtn.wasClicked(); // Eat up result
            for (int r = 0; r < 3; r++)
                for (int c = 0; c < 3; c++)
                {
                    tiles[r][c] = (display ? new PatternTile(solution[r][c]) : new PatternTile());
                    getWorld().addObject(tiles[r][c], getWorld().getWidth() / 2 - PatternTile.SIZE - 1 + (PatternTile.SIZE + 1) * c, getWorld().getHeight() / 2 - PatternTile.SIZE - 1 + (PatternTile.SIZE + 1) * r);
                }
            
            Terrain.main.record(display ? "The tiles here seem to just be a display." : "The tiles here seem to respond when left or right clicking them.");
        }
    }
}