import greenfoot.Actor;
import greenfoot.GreenfootImage;
import greenfoot.Greenfoot;

/**
 * PatternTile
 * 
 * @author Taylor Born
 * @version January 2014
 */
public class PatternTile extends Actor
{
    public static final int SIZE = 39;
    private static final GreenfootImage[] IMAGES = new GreenfootImage[7];
    private static final GreenfootImage[] SOLVED_IMAGES = new GreenfootImage[6];
    static
    {
        for (int i = 0; i < IMAGES.length; i++)
        {
            IMAGES[i] = new GreenfootImage("Pattern" + (char)('A' + i) + ".png");
            if (i != 0)
                SOLVED_IMAGES[i - 1] = new GreenfootImage("Pattern" + (char)('A' + i) + "_Solved.png");
        }
    }
    private static final int FADE_IN    = 10;
    private static final int FADE_OUT   = -10;
    private static final int FADE_NONE  = 0;

    private int fade = FADE_IN;
    private int state = -1;

    public PatternTile()
    {
        state = 0;
        initImage();
        update(0);
    }
    public PatternTile(int state)
    {
        initImage();
        if (state == 0)
            getImage().drawImage(IMAGES[0], 0, 0);
        else
        {
            getImage().drawImage(SOLVED_IMAGES[(state - 1) / 4], 0, 0);
            setRotation(((state - 1) % 4) * 90);
        }
        // Leave my state at -1 to indicate locked
    }
    private void initImage()
    {
        setImage(new GreenfootImage(SIZE, SIZE));
        getImage().setTransparency(5);
    }
    
    /**
     * To be called when Patterns are as input
     */
    public int getState()
    {
        return state;
    }
    
    /**
     * To be called when Patterns are as input. And when accept all Pattern's state
     */
    public void lock()
    {
        if (state != 0)
        {
            getImage().clear();
            getImage().drawImage(SOLVED_IMAGES[(state - 1) / 4], 0, 0);
        }
        state = -1;
        fade = FADE_OUT;
    }
    
    private void update(int s)
    {
        getImage().clear();
        if (s == 0)
        {
            getImage().drawImage(IMAGES[0], 0, 0);
            setRotation(0);
        }
        else
        {
            getImage().drawImage(IMAGES[1 + (s - 1) / 4], 0, 0);
            setRotation(((s - 1) % 4) * 90);
        }
    }
    
    public void fadeOut()
    {
        state = -1;
        fade = FADE_OUT;
    }
    
    @Override
    public void act()
    {
        if (fade != FADE_NONE)
        {
            int transparency = getImage().getTransparency();
            getImage().setTransparency(transparency + fade);
            if (transparency + fade == 255)
                fade = FADE_NONE;
            else if (transparency + fade == 5)
                getWorld().removeObject(this);
            return;
        }
        
        if (state == -1)
            return;
        
        if (Greenfoot.mousePressed(this))
        {
            int btn = Greenfoot.getMouseInfo().getButton();
            if (btn == 1)
            {
                if (state == 0)
                    state = 1;
                else
                {
                    state = 1 + ((state - 1) / 4 + 1) * 4;
                    if (state == 1 + (IMAGES.length - 1) * 4)
                        state = 0;
                }
                update(state);
            }
            else if (btn == 3 && state != 0)
            {
                int rotation = getRotation() + 90;
                if (rotation == 360)
                    rotation = 0;
                setRotation(rotation);
                
                state = 1 + ((state - 1) / 4) * 4 + rotation / 90;
            }
        }
    }
}