import greenfoot.GreenfootImage;
import greenfoot.World;
import greenfoot.Actor;
import java.awt.Color;

/**
 * Projectile
 * 
 * @author Taylor Born
 * @version May 2012
 */
public class Projectile extends GameObject
{
    private int origX, origY;
    private double sX, sY;
    private final int speed = 11;

    private Projectile(int r)
    {
        setRotation(r);
        GreenfootImage pic = new GreenfootImage(5, 3);
        pic.fill();
        pic.setColor(Color.WHITE);
        pic.setColorAt(2, 1, Color.WHITE);
        setImage(pic);
    }
    public Projectile(int r, double s, int r2)
    {
        this(r2);
        sX = Math.cos(Math.toRadians(r)) * s + Math.cos(Math.toRadians(r2)) * speed;
        sY = Math.sin(Math.toRadians(r)) * s + Math.sin(Math.toRadians(r2)) * speed;
    }
    public Projectile(int xM, int yM, double s, int r)
    {
        this(r);
        sX = xM * s + Math.cos(Math.toRadians(r)) * speed;
        sY = yM * s + Math.sin(Math.toRadians(r)) * speed;
    }
    
    @Override
    public void act()
    {
        translate(sX, sY);
        check();
    }
    
    private void check()
    {
        double distance = distanceFrom(origX, origY);
        if (distance > 100 || Terrain.main.isOut(this) || hitsGate(3))
            getWorld().removeObject(this);
        else
        {
            if (distance > 50)
                getImage().setTransparency((int)((1 - (distance - 50) / 50d) * 255));
            Soldier soldier = (Soldier)getOneIntersectingObject(Soldier.class);
            if (soldier != null && soldier != Terrain.main.getPlayer())
            {
                getWorld().removeObject(this);
                soldier.die();
            }
        }
    }
    
    @Override
    public void addedToWorld(World world)
    {
        super.addedToWorld(world);
        origX = getX();
        origY = getY();
        check();
    }
}