import greenfoot.Greenfoot;
import java.awt.Point;
import java.util.List;

/**
 * Soldier
 * 
 * @author Taylor Born
 * @version May 2012
 */
public class Soldier extends Unit
{
    private static final int ROTATION_SPEED = 3;
    private static final double ACCELERATION = .15;
    private static final double MAX_SPEED = 2.5;
    
    private char keyID;
    
    public void giveKey(char keyID)
    {
        this.keyID = keyID;
    }
    public void die()
    {
        if (keyID != 0)
            getWorld().addObject(new Key(keyID), getX(), getY());
        getWorld().removeObject(this);
    }
    
    private boolean moveByMouse = false;

    protected void handlePlayer()
    {
        speed = MAX_SPEED;
        if (Greenfoot.isKeyDown("w"))
            moveInDirection(270);
        else if (Greenfoot.isKeyDown("s"))
            moveInDirection(90);
        if (Greenfoot.isKeyDown("a"))
            moveInDirection(180);
        else if (Greenfoot.isKeyDown("d"))
            moveInDirection(0);
        Point p = ((Terrain)getWorld()).getLastMouse();
        turnTowards((int)p.getX(), (int)p.getY());
        
        if (Greenfoot.mousePressed(null) && Greenfoot.getMouseInfo().getButton() == 3)
            moveByMouse = true;
        if (Greenfoot.mouseClicked(null))
            moveByMouse = false;
        if (moveByMouse)
        {
            Point mouse = Terrain.main.getLastMouse();
            double a = Math.atan2(mouse.getY() - getY(), mouse.getX() - getX());
            while (a < 0)
                a += Math.PI * 2;
            while (a > Math.PI * 2)
                a -= Math.PI * 2;
            int direction = 0;
            if (a > Math.PI / 4)
                if (a <= Math.PI * 3 / 4)
                    direction = 90;
                else if (a <= Math.PI * 5 / 4)
                    direction = 180;
                else if (a <= Math.PI * 7 / 4)
                    direction = 270;
            moveInDirection(direction);
        }
        
        speed = 0;
        
        super.handlePlayer();
        
//         if (KeysPressed.testKey("r"))
//         {
//             Vehicle v = (Vehicle)getOneIntersectingObject(Vehicle.class);
//             if (v != null)
//                 Terrain.main.enterVehicle(v);
//         }
    }
    protected void fire()
    {
        if (isPlayer())
            getWorld().addObject(new Projectile(Greenfoot.isKeyDown("a") ? -1 : Greenfoot.isKeyDown("d") ? 1 : 0, Greenfoot.isKeyDown("w") ? -1 : Greenfoot.isKeyDown("s") ? 1 : 0, MAX_SPEED, getRotation()), getX() + (int)(Math.cos(Math.toRadians(getRotation())) * (-14 + 26) + Math.cos(Math.toRadians(getRotation() + 90)) * (-14 + 19)), getY() + (int)(Math.sin(Math.toRadians(getRotation() + 90)) * (-14 + 19) + Math.sin(Math.toRadians(getRotation())) * (-14 + 26)));
        else
            getWorld().addObject(new Projectile(getRotation(), speed, getRotation()), getX() + (int)(Math.cos(Math.toRadians(getRotation())) * (-14 + 26) + Math.cos(Math.toRadians(getRotation() + 90)) * (-14 + 19)), getY() + (int)(Math.sin(Math.toRadians(getRotation() + 90)) * (-14 + 19) + Math.sin(Math.toRadians(getRotation())) * (-14 + 26)));
    }
    protected void go()
    {
        
    }
    protected void slowDown()
    {
        if (Math.abs(speed) < .1)
            speed = 0;
        else if (speed > 0)
            speed -= speed / 10;
        else if (speed < 0)
            speed += -speed / 10;
    }
    protected boolean canTurnWhenStill()
    {
        return true;
    }
    protected int getRotationSpeed()
    {
        return ROTATION_SPEED;
    }
    protected double getAcceleration()
    {
        return ACCELERATION;
    }
    protected double getMaxSpeed()
    {
        return MAX_SPEED;
    }
    protected boolean hitsSomething()
    {
        if (super.hitsSomething() || hitsTree() || Terrain.main.isOut(this) || hitsGate(5))// || hitsLandUnits()
            return true;
        
        if (!Terrain.main.isWater(this))
            return false;
        for (Transport transport : (List<Transport>)getWorld().getObjects(Transport.class))
            if (transport.onThis(this))
                return false;
        return true;
    }
}