import greenfoot.World;
import greenfoot.GreenfootImage;
import greenfoot.Greenfoot;
import greenfoot.MouseInfo;
import greenfoot.Actor;
import greenfoot.UserInfo;
import java.awt.Point;
import java.awt.Graphics2D;
import java.awt.Color;
import java.awt.AlphaComposite;
import java.util.List;
import java.util.ArrayList;
import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.io.IOException;

/**
 * Terrain
 * 
 * @author Taylor Born
 * @version May 2012 - January 2014
 */
public class Terrain extends World
{
    public static Terrain main;
    public static final int WIDTH = 600;
    public static final int HEIGHT = 400;
    public static final int CENTER_PLAYER_X = WIDTH - (WIDTH - 200) / 2;
    public static final int CENTER_PLAYER_Y = HEIGHT / 2;
    private static final GreenfootImage GRASS = new GreenfootImage("Grass.jpg");
    private static GreenfootImage[][] grass = new GreenfootImage[17][17];
    static
    {
        GreenfootImage g = new GreenfootImage("Grass.jpg");
        for (int r = 0; r < grass.length; r++)
            for (int c = 0; c < grass.length; c++)
            {
                grass[r][c] = new GreenfootImage(30, 30);
                grass[r][c].drawImage(g, -c * 30, -r * 30);
            }
        GUI_Component.setDefaultColors(new Color(0, 0, 0, 180), Color.WHITE, Color.WHITE, new Color(70, 70, 70, 180), Color.GRAY, Color.BLUE);
    }
    
    private static final GreenfootImage IMAGE = new GreenfootImage(WIDTH, HEIGHT);
    private boolean loggedIn = false;
    private TwoPlayerUserInfo twoPlayerUserInfo;// = new TwoPlayerUserInfo(WIDTH, HEIGHT, 1);
    private Button lobbyBtn = new Button("Lobby", new Point(42, 23));
    
    private static final int STATE_NULL     = 0; // Not paired
    private static final int STATE_START    = 1; // Are paired, need to send initial info
    private static final int STATE_READY    = 2; // Initial info is sent, waiting for other player's initial info
    private static final int STATE_PLAYING  = 3;
    
    private int gameState = STATE_NULL;
    
    private int randomSeed;
    private int pathSelected;
    private int level = 1;
    private MessageContents initialSetup;
    
    public static final int MESSAGE_TRANSPORT   = 0;
    
    public static final int ACCESS_KEY_LENGTH = 6;
    
    private Point lastMouse = new Point(-1, -1);
    private Unit player;
    private int screenX, screenY;
    private ArrayList<Map> maps = new ArrayList<Map>();
    private Map currentMap;
    private Fog fogCover = new Fog();
    private TextBox chatInput = new TextBox(new Point(200, 60), "");
    private ChatRecord chatRecord = new ChatRecord(200, HEIGHT - 60 - 10);
    private Button[] choiceBtns = new Button[]{ new Button("Volunteer for Upper Wing", new Point(150, 23)), new Button("Volunteer for Lower Wing", new Point(150, 23)) };
    private int lastNoticeGiven;
    
    private String chatMessageToSend;
    private boolean sendingChatMessage;
    
    private static final boolean TESTING_OFFSITE = false; /* */

    public Terrain()
    {
        super(WIDTH, HEIGHT, 1, false);
        main = this;
        setPaintOrder(GUI_Component.class, PatternTile.class, Fog.class, Soldier.class, Gate.class, Fence.class, TreePart.class, Tree.class, Transport.class);
        IMAGE.fill();
        setBackground(IMAGE);
        initiateMaps();
        chatInput.setMaxLength(250);
        chatInput.dontAccept("\n");// \\"); // The slash is being used as backspace as well, for mac users =)
        
        if (!UserInfo.isStorageAvailable())
        {
            addObject(new Actor(){{ setImage(new GreenfootImage("  Please login to site,  \n  UserInfo required.  ", 30, Color.WHITE, Color.BLACK)); }}, WIDTH / 2, HEIGHT / 2);
            return;
        }
        
        /* */
//         String name = UserInfo.getMyInfo().getUserName();
//         if (!name.equals("bourne") && !name.equals("SuperSnakes"))
//             return;
        /* */
        loggedIn = true;
        
        twoPlayerUserInfo = new TwoPlayerUserInfo(WIDTH, HEIGHT, 1);
        addObject(lobbyBtn, WIDTH - lobbyBtn.getGUIWidth() / 2, lobbyBtn.getGUIHeight() / 2);
//         twoPlayerUserInfo.enterLobby(this);
        
        if (TESTING_OFFSITE) /* */
            newGame(0); /* */
    }
    private void initiateMaps()
    {
        try
        {
            BufferedReader br = new BufferedReader(new InputStreamReader(getClass().getClassLoader().getResource("Maps.txt").openStream()));
            String str = "";
            while (str != null)
            {
                if (!str.trim().isEmpty()) // if (str.charAt(0) == '%')
                    maps.add(new Map(br, str));
                str = br.readLine();
            }
            br.close();
        }
        catch (IOException e)
        {}
    }
    private void newGame(int map)
    {
        player = new Soldier();
        addObject(player, CENTER_PLAYER_X, CENTER_PLAYER_Y);
        player.rotate(-90);
        
        screenX = 0;
        screenY = 0;
        loadMap(map);
        
        updateGrass();
        fogCover.update();
        addObject(fogCover, WIDTH / 2, HEIGHT / 2);
        
        gameState = STATE_PLAYING;
    }
    private void loadMap(int i)
    {
        currentMap = maps.get(i);
        currentMap.load(this);
    }
    public boolean isPaused()
    {
        return gameState == STATE_NULL || TextBox.textBoxHasFocus() || !getObjects(PatternTile.class).isEmpty() || !twoPlayerUserInfo.isPaired();
    }
    
    public boolean isOut(int x, int y)
    {
        return currentMap.isOut(-(int)screenX + x - CENTER_PLAYER_X, -(int)screenY + y - CENTER_PLAYER_Y);
    }
    public boolean isOut(Actor actor)
    {
        return isOut(actor.getX(), actor.getY());
    }
    public boolean isWater(Actor actor)
    {
        return currentMap.isWater(-(int)screenX + actor.getX() - CENTER_PLAYER_X, -(int)screenY + actor.getY() - CENTER_PLAYER_Y);
    }
    
    public void pickedUpKey(char id)
    {
        chatRecord.record("You Acquired Access Key " + accessKeyPrefix(false) + "-" + id + ": " + getKey(id, false));
    }
    public char accessKeyPrefix(boolean forMe)
    {
        return (forMe ? pathSelected : 1 - pathSelected) == 0 ? 'A' : 'B';
    }
    public String getKey(char id, boolean forMe)
    {
        RandomGenerator.setSeed(randomSeed + level * 10000 + (forMe ? pathSelected : 1 - pathSelected) * 1500 + id);
        String key = "";
        while (key.length() < ACCESS_KEY_LENGTH)
            key += (char)('A' + RandomGenerator.getRandom(0, 25));
        return key;
    }
    public int[][] getPattern(char id)
    {
        RandomGenerator.setSeed(randomSeed + level * 15000 + id);
        int[][] solution = new int[3][3];
        for (int r = 0; r < 3; r++)
            for (int c = 0; c < 3; c++)
                solution[r][c] = (RandomGenerator.getRandom(0, 8) < 3 ? 0 : RandomGenerator.getRandom(0, 6 * 4));
        return solution;
    }
    public void record(String text)
    {
        chatRecord.record(text);
    }
    
    @Override
    public void act()
    {
        if (!loggedIn)
            return;
        
        if (lobbyBtn.wasClicked())
        {
            removeObjects(getObjects(null));
            addObject(lobbyBtn, WIDTH - lobbyBtn.getGUIWidth() / 2, lobbyBtn.getGUIHeight() / 2);
            IMAGE.fill();
            gameState = STATE_NULL;
            twoPlayerUserInfo.enterLobby(this);
            return;
        }
        
        if (!TESTING_OFFSITE) /* */
        if (gameState != STATE_NULL && !twoPlayerUserInfo.isPaired())
        {
            gameState = STATE_NULL;
            chatInput.removeFromWorld();
            chatRecord.record("No longer paired with " + twoPlayerUserInfo.getOpponentName() + ". They left. Game over.");
            return;
        }
        
        switch (gameState)
        {
            case STATE_NULL:
                if (twoPlayerUserInfo.isPaired())
                    nowPaired();
                break;
            case STATE_START:
                if (isReadyToSend())
                {
                    for (int i = 0; i < 2; i++)
                        if (choiceBtns[i].wasClicked())
                        {
                            pathSelected = i;
                            removeObject(choiceBtns[0]);
                            removeObject(choiceBtns[1]);
                        }
                    if (pathSelected == -1)
                        break;
                    randomSeed = (int)(Math.random() * 100);
                    sendMessage(new Integer[]{ new Integer(randomSeed), new Integer(pathSelected) }, null);
                    gameState = STATE_READY;
                    checkSetupGame();
                }
                break;
            case STATE_PLAYING:
                centerOnPlayer();
                updateGrass();
                fogCover.update();
        }
        
        listenForInputDevices();
        
        // Following should only happen when gameState is not STATE_NULL
        if (gameState == STATE_NULL)
            return;
        
        handleSendingChat();
        handleNotices();
        handleMessages();
    }
    private void nowPaired()
    {
        chatMessageToSend = null;
        sendingChatMessage = false;
        lastNoticeGiven = 0;
        pathSelected = -1;
        initialSetup = null;
        chatRecord.clear();
        addObject(chatRecord, chatRecord.getGUIWidth() / 2, chatRecord.getGUIHeight() / 2);
        chatRecord.record("Paired with " + twoPlayerUserInfo.getOpponentName() + '.');
        chatRecord.record("Mission: Infiltrate the enemy base. Take down the commander.", true, "HQ", false);
        chatRecord.record("One of you take the Upper Wing, the other take the Lower Wing. Keep in radio contact with each other.", true, "HQ", false);
        chatRecord.record("HQ out.", true, "HQ", false);
        choiceBtns[0].wasClicked(); // Eat up result
        choiceBtns[1].wasClicked(); // Eat up result
        addObject(choiceBtns[0], CENTER_PLAYER_X, CENTER_PLAYER_Y - 30);
        addObject(choiceBtns[1], CENTER_PLAYER_X, CENTER_PLAYER_Y + 30);
        chatInput.clear();
        addObject(chatInput, chatInput.getGUIWidth() / 2, HEIGHT - chatInput.getGUIHeight() / 2);
        gameState = STATE_START;
    }
    /**
     * Takes the place of TwoPlayerUserInfo.isReadyToSend(), so know when we have sent a chat message, if it has gone through.
     */
    public boolean isReadyToSend()
    {
        if (!twoPlayerUserInfo.isReadyToSend())
            return false;
        if (sendingChatMessage)
        {
            sendingChatMessage = false;
            chatInput.clear();
            addObject(chatInput, chatInput.getGUIWidth() / 2, HEIGHT - chatInput.getGUIHeight() / 2);
        }
        return true;
    }
    /**
     * Should only be called after isReadyToSend() returns true.
     * @see isReadyToSend()
     */
    public void sendMessage(Integer[] ints, String[] strings)
    {
        twoPlayerUserInfo.sendMessage(ints, strings);
    }
    private void handleSendingChat()
    {
        if (chatInput.inWorld())
            if ((chatInput.hasFocus() || (!TextBox.textBoxHasFocus() && getObjects(InputWindow.class).isEmpty())) && Greenfoot.isKeyDown("enter"))
                if (chatInput.hasFocus())
                {
                    if (!chatInput.getText().isEmpty())
                    {
                        chatMessageToSend = chatInput.getText();
                        chatInput.removeFromWorld();
                    }
                }
                else
                    chatInput.giveFocus();
        if (isReadyToSend() && chatMessageToSend != null) // Order isReadyToSend() first, so there is an active listener on if a sent chat message has gone through
        {
            chatRecord.record(chatInput.getText(), false, twoPlayerUserInfo.getMyName(), true);
            
            String[] strings = new String[5];
            for (int i = 0; i < strings.length; i++)
                if (chatMessageToSend.length() <= 50)
                {
                    strings[i] = chatMessageToSend;
                    chatMessageToSend = "";
                }
                else
                {
                    strings[i] = chatMessageToSend.substring(0, 50);
                    chatMessageToSend = chatMessageToSend.substring(50);
                }
            sendMessage(null, strings);
            sendingChatMessage = true;
            chatMessageToSend = null;
        }
        
    }
    private void checkSetupGame()
    {
        if (initialSetup == null || gameState != STATE_READY)
            return;
        
        if (pathSelected == initialSetup.getInt(1))
            if (randomSeed < initialSetup.getInt(0) || (randomSeed == initialSetup.getInt(0) && twoPlayerUserInfo.getMyName().compareTo(twoPlayerUserInfo.getOpponentName()) < 0))
                pathSelected = 1 - pathSelected;
        randomSeed += initialSetup.getInt(0);
        
        chatRecord.record("I got " + (1 - pathSelected == 0 ? "Upper" : "Lower") + " Wing.", true, twoPlayerUserInfo.getOpponentName(), true);
        chatRecord.record("I got " + (pathSelected == 0 ? "Upper" : "Lower") + " Wing.", true, twoPlayerUserInfo.getMyName(), true);
        
        newGame(pathSelected);
    }
    private void listenForInputDevices()
    {
        KeysPressed.update();
        MouseInfo mouse = Greenfoot.getMouseInfo();
        if (Greenfoot.mouseMoved(null) || Greenfoot.mouseDragged(null))
            lastMouse = new Point(mouse.getX(), mouse.getY());
    }
    private void handleNotices()
    {
        int seconds = twoPlayerUserInfo.getTimeSinceOpponentLastActivity();
        if (seconds > 20)
        {
            if (lastNoticeGiven == 0 || seconds - lastNoticeGiven > 60)
            {
                chatRecord.record("**Notice** " + twoPlayerUserInfo.getOpponentName() + " has been inactive for " + twoPlayerUserInfo.getTimeSinceOpponentLastActivityAsString() + ". They may have paused the scenario, or left.");
                lastNoticeGiven = seconds;
            }
        }
        else if (lastNoticeGiven != 0)
        {
            chatRecord.record("**Notice** " + twoPlayerUserInfo.getOpponentName() + " has been active again recently.");
            lastNoticeGiven = 0;
        }
    }
    private void handleMessages()
    {
        if (!twoPlayerUserInfo.hasMessage())
            return;
        MessageContents contents = twoPlayerUserInfo.getMessage();
        if (contents.hasString(0))
        {
            String text = "";
            for (int i = 0; i < 5; i++)
                text += contents.getString(i);
            chatRecord.record(text, false, twoPlayerUserInfo.getOpponentName(), true);
        }
        else if (gameState <= STATE_READY)
        {
            initialSetup = contents;
            checkSetupGame();
        }
        else //if (gameState == STATE_PLAYING)
        {
            switch (contents.getInt(0))
            {
                case MESSAGE_TRANSPORT:
                    char id = (char)contents.getInt(1);
                    boolean headHome = contents.getInt(2) == 1;
                    for (Transport transport : (List<Transport>)getObjects(Transport.class))
                        if (transport.getID() == id)
                            transport.headTo(headHome);
                    break;
                
            }
        }
    }
    
    public Point getLastMouse()
    {
        return lastMouse;
    }
    private void centerOnPlayer()
    {
        int x = CENTER_PLAYER_X - player.getX();
        int y = CENTER_PLAYER_Y - player.getY();
        if (x == 0 && y == 0)
            return;
        screenX += x;
        screenY += y;
        for (GameObject go : (List<GameObject>)getObjects(GameObject.class))
            go.translate(x, y);
    }
    public void updateGrass()
    {
        int originX = -((-(int)screenX - CENTER_PLAYER_X) % 30) - 15 - 30;
        int originY = -((-(int)screenY - CENTER_PLAYER_Y) % 30) - 15 - 30;
        IMAGE.fill();
        for (int y = originY; y <= HEIGHT; y += 30)
            for (int x = originX; x <= WIDTH; x += 30)
                if (!isOut(x, y))
                    IMAGE.drawImage(grass[(-screenY + 30000 + y) / 30 % grass.length][(-screenX + 30000 + x) / 30 % grass[0].length], x, y);
        
//         for (int i = -GRASS.getWidth() + (int)(screenX % GRASS.getWidth()); i < WIDTH; i += GRASS.getWidth())
//             for (int k = -GRASS.getHeight() + (int)(screenY % GRASS.getHeight()); k < HEIGHT; k += GRASS.getHeight())
//                 IMAGE.drawImage(GRASS, i, k);
    }
    public Unit getPlayer()
    {
        return player;
    }
    public void exitVehicle(int x, int y)
    {
        int r = player.getRotation();
        player = new Soldier();
        player.setRotation(r);
        addObject(player, x, y);
    }
//     public void enterVehicle(Vehicle v)
//     {
//         removeObject(player);
//         player = v;
//     }
    
    @Override
    public void started()
    {
        twoPlayerUserInfo.scenarioStarted();
    }
    @Override
    public void stopped()
    {
        twoPlayerUserInfo.scenearioStopped();
    }
    
    private class Fog extends Actor
    {
        public Fog()
        {
            setImage(new GreenfootImage(WIDTH, HEIGHT));
        }
        public void update()
        {
//             getImage().fill(); // Or fill with a lightly transparent black
//             Graphics2D gr = getImage().getAwtImage().createGraphics();
//             gr.setColor(new Color(0, 0, 0, 0));
//             gr.setComposite(AlphaComposite.getInstance(AlphaComposite.DST_IN, 1f));
//             
//             
//             gr.dispose();
            
            

            // NOT DOING THIS:
            
//             getImage().clear();
//             int originX = -((-(int)screenX - CENTER_PLAYER_X) % 30) - 15 - 30;
//             int originY = -((-(int)screenY - CENTER_PLAYER_Y) % 30) - 15 - 30;
//             for (int y = originY; y <= HEIGHT; y += 30)
//                 for (int x = originX; x <= WIDTH; x += 30)
//                     if (isOut(x, y))
//                         getImage().fillRect(x, y, 30, 30);
        }
    }
}