import greenfoot.GreenfootImage;
import greenfoot.World;
import greenfoot.Actor;
import java.awt.Color;
import java.util.List;

/**
 * Transport
 * 
 * @author Taylor Born
 * @version January 2014
 */
public class Transport extends GameObject
{
    private static GreenfootImage IMAGE = new GreenfootImage("Transport.png");

    private char id;
    private double[] home;
    private int[] offset;
    private boolean setForHome = true;

    public Transport(char id, int offsetX, int offsetY)
    {
        this.id = id;
        offset = new int[]{ offsetX * 30, offsetY * 30 };
        setImage(IMAGE);
    }
    
    public char getID()
    {
        return id;
    }
    
    @Override
    public void act()
    {
        double headingToX = home[0] + (setForHome ? 0 : offset[0]);
        double headingToY = home[1] + (setForHome ? 0 : offset[1]);
        
        int amtX = 0;
        if (Math.abs(headingToX - getX()) >= 1)
            amtX = headingToX > getX() ? 1 : -1;
        int amtY = 0;
        if (Math.abs(headingToY - getY()) >= 1)
            amtY = headingToY > getY() ? 1 : -1;
        transport(amtX, amtY);
        
//         double distance = distanceFrom(headingToX, headingToY);
//         if (distance > 1)
//         {
//             double a = Math.atan2(headingToY - getY(), headingToX - getX());
//             transport(Math.cos(a), Math.sin(a));
//         }
//         else
//             transport(headingToX - x, headingToY - y);
    }
    
    private void transport(double x, double y)
    {
        for (Soldier soldier : (List<Soldier>)getWorld().getObjects(Soldier.class))
            if (onThis(soldier))
                soldier.translate(x, y);
        super.translate(x, y);
    }
    
    public void headTo(boolean home)
    {
        setForHome = home;
    }
    
    /**
     * Important that this overrided method is not used to move the Transport, only to translate for scrolling.
     */
    @Override
    public void translate(double a, double b)
    {
        super.translate(a, b);
        home[0] += a;
        home[1] += b;
    }
    
    @Override
    public void addedToWorld(World world)
    {
        super.addedToWorld(world);
        home = new double[]{ x, y };
    }
}