import greenfoot.Greenfoot;
import java.awt.Point;

/**
 * Unit
 * 
 * @author Taylor Born
 * @version May 2012
 */
public abstract class Unit extends GameObject
{
    protected double speed;

    public void act()
    {
        if (Terrain.main.isPaused())
            return;
        
        if (isPlayer())
            handlePlayer();
        else
            go();
        if (getWorld() != null)
            move();
    }
    protected void handlePlayer()
    {
        if (KeysPressed.testKey("space") || (Greenfoot.mousePressed(null) && Greenfoot.getMouseInfo().getButton() == 1))
            fire();
    }
    protected boolean isPlayer()
    {
        return this == ((Terrain)getWorld()).getPlayer();
    }
    
    protected abstract void fire();
    
    protected abstract void go();
    
    protected abstract int getRotationSpeed();
    
    protected abstract double getAcceleration();
    
    protected abstract double getMaxSpeed();
    
    protected boolean canTurnWhenStill()
    {
        return false;
    }
    protected void accelerate(boolean forward)
    {
        if (forward)
        {
            if (speed >= 0)
            {
                speed += getAcceleration();
                if (speed > getMaxSpeed())
                    speed = getMaxSpeed();
            }
            else
                speed += getAcceleration() / 2;
        }
        else if (speed <= 0)
        {
            speed -= getAcceleration() / 2;
            if (speed < -getMaxSpeed() / 2)
                speed = -getMaxSpeed() / 2;
        }
        else
            speed -= getAcceleration() / 2;
    }
    
    protected abstract void slowDown();
    
    protected void move()
    {
        moveInDirection(getRotation());
    }
    protected void moveInDirection(int r)
    {
        double rad = Math.toRadians(r);
        
        double cosAmt = Math.cos(rad);
        translate(cosAmt * speed, 0);
        if (hitsSomething())
            translate(cosAmt * -speed, 0);
        
        double sinAmt = Math.sin(rad);
        translate(0, sinAmt * speed);
        if (hitsSomething())
            translate(0, sinAmt * -speed);
    }
    
    protected boolean hitsSomething()
    {
//         for (Object o : getWorld().getObjects(Sector.class))
//         {
//             Sector s = (Sector)o;
//             if (getX() >= s.getX() - Sector.SIZE / 2 && getX() <= s.getX() + Sector.SIZE / 2 && getY() >= s.getY() - Sector.SIZE / 2 && getY() <= s.getY() + Sector.SIZE / 2)
//                 return false;
//         }
        return false;
    }
    
    protected boolean hitsTree()
    {
//         return !getNeighbours(getImage().getWidth() / 2, true, Tree.class).isEmpty();
        return getOneIntersectingObject(Tree.class) != null;
    }
//     protected boolean hitsLandUnits()
//     {
// //         for (Object o : getWorld().getObjects(LandUnit.class))
// //         {
// //             if (o != this)
// //             {
// //                 Unit u = (Unit)o;
// //                 
// //                 if (withinDistance(u.getX(), u.getY(), getImage().getWidth() / 2))
// //                     return true;
// //             }
// //         }
//         return false;
//     }
//     protected boolean hitsAirUnits()
//     {
//         for (Object o : getWorld().getObjects(Helicopter.class))
//         {
//             if (o != this)
//             {
//                 Helicopter h = (Helicopter)o;
//                 
//                 if (withinDistance(h.getX(), h.getY(), getImage().getWidth() / 2))
//                     return true;
//             }
//         }
//         return false;
//     }
    protected void turnRight()
    {
        rotate(getRotationSpeed());
    }
    protected void turnLeft()
    {
        rotate(-getRotationSpeed());
    }
    public void rotate(int r)
    {
        setRotation(getRotation() + r);
    }
}
